/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.tree.TreePath;

import org.w3c.dom.Node;

import ch.cern.lhcb.xmleditor.specific.XmlDisplay;

/**
 * this implements the menu of the TreeView window
 * 
 * @author Sebastien Ponce
 */
public class TreeViewMenu extends JMenuBar implements ItemListener,
        ActionListener {

    /*
     * Strings used for the different items of the menus
     */
    private static final String FILE_MENU = "File";
    private static final String SAVE_ITEM = "Save";
    private static final String CLOSE_ITEM = "Close";
    private static final String OPTION_MENU = "Option";
    private static final String VALUE_AS_ATTRIBUTE_ITEM = "Value as attribute";
    private static final String DISPLAY_COMMENTS_ITEM = "Display Comments";
    private static final String ATTRIBUTE_DISPLAYING_ITEM = "Attribute Displaying";
    private static final String SUBELEMENT_DISPLAYING_ITEM = "SubElement Displaying";
    private static final String NAME_DISPLAYING_ITEM = "Element Name Displaying";

    /*
     * Some of the menu items
     */
    private JCheckBoxMenuItem vAsAMenuItem;
    private JCheckBoxMenuItem displayCommentsMenuItem;
    private JMenuItem attrDisMenuItem;
    private JMenuItem subElemDisMenuItem;
    private JMenuItem nameDisMenuItem;

    /*
     * the underlying treeView
     */
    private TreeView treeView;

    /**
     * constructor : creates the menu bar
     * 
     * @param treeView the underlying TreeView
     */
    public TreeViewMenu(TreeView treeView) {
        super();
        this.treeView = treeView;

        // File Menu
        ////////////
        // creates the File menu, with mnemonic F
        JMenu fileMenu = new JMenu(FILE_MENU);
        fileMenu.setMnemonic(KeyEvent.VK_F);
        // creates some items and their mnemonics
        JMenuItem saveMenuItem = new JMenuItem(SAVE_ITEM);
        saveMenuItem.setMnemonic(KeyEvent.VK_S);
        JMenuItem closeMenuItem = new JMenuItem(CLOSE_ITEM);
        closeMenuItem.setMnemonic(KeyEvent.VK_C);
        // adds a listener to each menuItem
        saveMenuItem.addActionListener(this);
        closeMenuItem.addActionListener(this);
        // builds the menu
        fileMenu.add(saveMenuItem);
        fileMenu.add(closeMenuItem);
        this.add(fileMenu);

        // Option Menu
        //////////////
        // creates the Option menu, with mnemonic O
        JMenu optionMenu = new JMenu(OPTION_MENU);
        optionMenu.setMnemonic(KeyEvent.VK_O);
        // creates some items and their mnemonics
        vAsAMenuItem = new JCheckBoxMenuItem(VALUE_AS_ATTRIBUTE_ITEM);
        vAsAMenuItem.setSelected(true);
        vAsAMenuItem.setMnemonic(KeyEvent.VK_L);
        displayCommentsMenuItem = new JCheckBoxMenuItem(DISPLAY_COMMENTS_ITEM);
        displayCommentsMenuItem.setSelected(false);
        displayCommentsMenuItem.setMnemonic(KeyEvent.VK_C);
        attrDisMenuItem = new JMenuItem(ATTRIBUTE_DISPLAYING_ITEM);
        attrDisMenuItem.setMnemonic(KeyEvent.VK_A);
        subElemDisMenuItem = new JMenuItem(SUBELEMENT_DISPLAYING_ITEM);
        subElemDisMenuItem.setMnemonic(KeyEvent.VK_S);
        nameDisMenuItem = new JMenuItem(NAME_DISPLAYING_ITEM);
        nameDisMenuItem.setMnemonic(KeyEvent.VK_N);
        // adds a listener to each menuItem and to the menu
        vAsAMenuItem.addItemListener(this);
        optionMenu.addItemListener(this);
        displayCommentsMenuItem.addItemListener(this);
        attrDisMenuItem.addActionListener(this);
        subElemDisMenuItem.addActionListener(this);
        nameDisMenuItem.addActionListener(this);
        // builds the menu
        optionMenu.add(vAsAMenuItem);
        optionMenu.add(displayCommentsMenuItem);
        optionMenu.addSeparator();
        optionMenu.add(attrDisMenuItem);
        optionMenu.add(subElemDisMenuItem);
        optionMenu.add(nameDisMenuItem);
        this.add(optionMenu);
    }

    /**
     * deals with item events occured on the main window
     * 
     * @param e an event
     */
    public void actionPerformed(ActionEvent e) {
        String item = ((JMenuItem) e.getSource()).getText();
        if (item.equals(SAVE_ITEM)) {
            treeView.save();
            return;
        }
        if (item.equals(CLOSE_ITEM)) {
            treeView.exit();
            return;
        }
        if (item.equals(ATTRIBUTE_DISPLAYING_ITEM)
                || item.equals(SUBELEMENT_DISPLAYING_ITEM)
                || item.equals(NAME_DISPLAYING_ITEM)) {
            XmlDisplay xmlDisplay = this.getSelectedXmlDisplay();
            // the XmlDisplay can't be null here since the menu would have been
            // disabled in such a case
            AbstractDialog dialog;
            if (item.equals(ATTRIBUTE_DISPLAYING_ITEM)) {
                dialog = new AttributeDialog(treeView.getEditor(), xmlDisplay);
            } else if (item.equals(SUBELEMENT_DISPLAYING_ITEM)) {
                dialog = new ElementDialog(treeView.getEditor(), xmlDisplay);
            } else if (item.equals(NAME_DISPLAYING_ITEM)) {
                dialog = new NameDialog(treeView.getEditor(), xmlDisplay);
            } else {
                return;
            }
            dialog.pack();
            dialog.setLocation(treeView.getLocation().x
                    + (treeView.getWidth() - dialog.getWidth()) / 2, treeView
                    .getLocation().y
                    + (treeView.getHeight() - dialog.getHeight()) / 2);
            dialog.setVisible(true);
            treeView.getEditor().redisplayAll();
            return;
        }
    }

    /**
     * deals with item events occured on the TreeView
     * 
     * @param e an event
     */
    public void itemStateChanged(ItemEvent e) {
        String item = ((JMenuItem) e.getSource()).getText();
        if (item.equals(OPTION_MENU)) {
            // here we enable or disable the attrDisMenuItem, subElemDisMenuItem
            // and nameDisMenuItem
            // depending of whether an XmlDisplay is available or not for
            // the currently selected nodes.
            // If none is available, we disable the menuItems
            // This last behavior is also valid for viewSourceMenuItem
            boolean xmlDisplayAvailable = this.getSelectedXmlDisplay() != null;
            attrDisMenuItem.setEnabled(xmlDisplayAvailable);
            subElemDisMenuItem.setEnabled(xmlDisplayAvailable);
            nameDisMenuItem.setEnabled(xmlDisplayAvailable);
        }
        if (item.equals(VALUE_AS_ATTRIBUTE_ITEM)) {
            // if value as attribute menuCheckboxItem was hit,
            // update the state of the selected window
            treeView.setValueDisplayedAsAttribute(vAsAMenuItem.isSelected());
            treeView.redisplayTree();
        }
        if (item.equals(DISPLAY_COMMENTS_ITEM)) {
            // if display comments menuCheckboxItem was hit,
            // update the state of the selected window
            treeView.setCommentDisplayed(displayCommentsMenuItem.isSelected());
            treeView.redisplayTree();
        }
    }

    /**
     * gets the XmlDisplay of the current set of selected node. If this sets
     * contents nodes related to different XmlDisplays, returns null. If one of
     * the selected nodes has no XmlDisplay, returns null;
     * 
     * @return the XmlDisplay for the selected nodes (if unique)
     */
    private XmlDisplay getSelectedXmlDisplay() {
        TreePath[] treePaths = treeView.getXmlTree().getSelectionPaths();
        XmlDisplay result = null;
        if (treePaths != null) {
            for (int i = 0; i < treePaths.length; i++) {
                Node node = ((XmlTreeNode) treePaths[i].getLastPathComponent())
                        .getNode();
                XmlDisplay display = XmlTree.getXmlDisplayFromNode(node);
                if (display == null) { return null; }
                if (result == null) {
                    result = display;
                } else {
                    if (display != result) { return null; }
                }
            }
        }
        return result;
    }

}