/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.gui;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Arrays;

import javax.swing.Icon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.tree.TreePath;

import org.w3c.dom.Node;

import ch.cern.lhcb.xmleditor.specific.XmlDisplay;

/**
 * this is an implementation of MouseListener that deals with the displaying of
 * popup menus on an XmlTree
 * 
 * @author Sebastien Ponce
 */
public class XmlMouseListener implements MouseListener {

    /**
     * texts on items of the popup menu
     */
    public final static String EDIT_REFERENCE = "edit original reference";
    public final static String CLOSE_REFERENCE = "close reference";
    public final static String REMOVE_ELEMENT = "remove this node";
    public final static String REMOVE_DEREF_ELEMENT = "remove reference on this node";
    public final static String REMOVE_ATTRIBUTE = "remove attribute";
    public final static String CREATE_ELEMENT = "Create new element";
    public final static String CREATE_ATTRIBUTE = "Create new attribute";
    public final static String SAVE_ITEM = "Save";
    public final static String COMMENT = "#comment";
    public final static String VALUE = "#value";
    public static final String VIEW_SOURCE_ITEM = "View source";

    /**
     * the Xml tree this listener is listening at
     */
    private XmlTree jTree;

    /**
     * constructor
     * 
     * @param jTree the XmlTree this listener will listen at
     */
    public XmlMouseListener(XmlTree jTree) {
        this.jTree = jTree;
    }

    /**
     * Invoked when the mouse has been clicked on a component.
     * @param e
     */
    public void mouseClicked(MouseEvent e) {
        // no action here
    }

    /**
     * Invoked when a mouse button has been pressed on a component.
     * @param e
     */
    // note that popup events are processed either in mousePressed
    // or in mouseReleased because it depends on the operating system
    public void mousePressed(MouseEvent e) {
        // only deals with popup events
        if (e.isPopupTrigger()) {
            this.showPopup(e);
        }
    }

    /**
     * Invoked when a mouse button has been released on a component.
     * @param e
     */
    // note that popup events are processed either in mousePressed
    // or in mouseReleased because it depends on the operating system
    public void mouseReleased(MouseEvent e) {
        // only deals with popup events
        if (e.isPopupTrigger()) {
            this.showPopup(e);
        }
    }

    /**
     * Invoked when the mouse enters a component.
     * @param e
     */
    public void mouseEntered(MouseEvent e) {
        // no action here
    }

    /**
     * Invoked when the mouse exits a component.
     * @param e
     */
    public void mouseExited(MouseEvent e) {
        // no action here
    }

    /**
     * this method displays, if needed, a popup menu adapted to the node clicked
     * 
     * @param e the mouse event concerned
     */
    private void showPopup(MouseEvent e) {
        // gets the treeNode that is concerned by this event and the underlying
        // node
        TreePath treePath = jTree.getClosestPathForLocation(e.getX(), e.getY());
        if (treePath == null) { return; }
        XmlTreeNode treeNode = (XmlTreeNode) treePath.getLastPathComponent();
        Node node = treeNode.getNode();
        // builds the popup menu and the listener
        JPopupMenu popupMenu = new JPopupMenu();
        XmlPopupListener popupListener = new XmlPopupListener(treeNode);
        // add some items to the popupmenu
        // first to remove the node (or the reference on it)
        JMenuItem removeItem;
        if (treeNode.getOldNode() != null) {
            removeItem = new JMenuItem(XmlMouseListener.REMOVE_DEREF_ELEMENT);
        } else {
            removeItem = new JMenuItem(XmlMouseListener.REMOVE_ELEMENT);
        }
        removeItem.addActionListener(popupListener);
        popupMenu.add(removeItem);
        // then to add new nodes or new attributes or to remove attributes
        // All that if we do not deal with a text node and if an XmlDisplay
        // exists
        if (node.getNodeType() != Node.TEXT_NODE
                && node.getNodeType() != Node.COMMENT_NODE) {
            XmlDisplay xmlDisplay = XmlTree.getXmlDisplayFromNode(node);
            if (xmlDisplay != null) {
                // retrieves the possible subelements
                String[] elementNames = xmlDisplay.getSubElementNames(node
                        .getNodeName());
                Arrays.sort(elementNames);
                // retrieves the possible attributes
                String[] attributeNames = xmlDisplay.getAttributeNames(node
                        .getNodeName());
                Arrays.sort(attributeNames);
                // Possible menus
                JMenu removeAttrMenu = null;
                JMenu createElemMenu = null;
                JMenu createAttrMenu = null;
                // if there are some subelements, fills the correponding menu
                boolean isSubElements = elementNames.length != 0
                        || xmlDisplay.elementHasValue(node.getNodeName());
                boolean isAttributes = attributeNames.length != 0;
                if (isSubElements) {
                    // creates the subMenu
                    createElemMenu = new JMenu(XmlMouseListener.CREATE_ELEMENT);
                    // for each possible subelement, creates an item
                    for (int i = 0; i < elementNames.length; i++) {
                        // gets an icon
                        Icon icon = jTree.getTreeView().getEditor()
                                .getIconsManager().getIcon(elementNames[i]);
                        // creates the item
                        JMenuItem item = new JMenuItem(elementNames[i], icon);
                        // add a listener on it
                        item.addActionListener(popupListener);
                        // add it to the menu
                        createElemMenu.add(item);
                    }
                    // does the same for comments if subelements are suthorized
                    if (elementNames.length != 0) {
                        Icon icon = jTree.getTreeView().getEditor()
                                .getIconsManager().getIcon(COMMENT);
                        JMenuItem item = new JMenuItem(
                                XmlMouseListener.COMMENT, icon);
                        item.addActionListener(popupListener);
                        createElemMenu.add(item);
                    }
                    // does the same for value if needed
                    if (xmlDisplay.elementHasValue(node.getNodeName())) {
                        Icon icon2 = jTree.getTreeView().getEditor()
                                .getIconsManager().getIcon(VALUE);
                        JMenuItem item2 = new JMenuItem(XmlMouseListener.VALUE,
                                icon2);
                        item2.addActionListener(popupListener);
                        createElemMenu.add(item2);
                    }
                }
                // if there are some attributes, fills the correponding menus
                if (isAttributes) {
                    // creates subMenus
                    removeAttrMenu = new JMenu(
                            XmlMouseListener.REMOVE_ATTRIBUTE);
                    createAttrMenu = new JMenu(
                            XmlMouseListener.CREATE_ATTRIBUTE);
                    // for each possible subelement, creates two items
                    for (int i = 0; i < attributeNames.length; i++) {
                        // creates the items
                        Icon icon = jTree.getTreeView().getEditor()
                                .getIconsManager().getIcon(attributeNames[i]);
                        JMenuItem addItem = new JMenuItem(attributeNames[i],
                                icon);
                        JMenuItem remItem = new JMenuItem(attributeNames[i],
                                icon);
                        // add listeners on them
                        addItem.addActionListener(popupListener);
                        remItem.addActionListener(popupListener);
                        // add them to the menus
                        createAttrMenu.add(addItem);
                        removeAttrMenu.add(remItem);
                    }
                }
                // if there are some attributes, displays the removeAttrMenu
                if (isAttributes) {
                    popupMenu.add(removeAttrMenu);
                }
                // if there are some subelements or attributes, adds a separator
                // in the menu
                if (isSubElements || isAttributes) {
                    popupMenu.addSeparator();
                }
                // if there are some subelements, displays the createElemMenu
                if (isSubElements) {
                    popupMenu.add(createElemMenu);
                }
                // if there are some attributes, displays the createAttrMenu
                if (isAttributes) {
                    popupMenu.add(createAttrMenu);
                }
            }
            // looks whether it was a reference in the past
            XmlTreeNode oldNode = treeNode.getOldNode();
            if (oldNode != null) {
                if (oldNode.isReference()) {
                    // if yes, add some items
                    JMenuItem closeRefItem = new JMenuItem(
                            XmlMouseListener.CLOSE_REFERENCE);
                    closeRefItem.addActionListener(popupListener);
                    JMenuItem editRefItem = new JMenuItem(
                            XmlMouseListener.EDIT_REFERENCE);
                    editRefItem.addActionListener(popupListener);
                    popupMenu.addSeparator();
                    popupMenu.add(closeRefItem);
                    popupMenu.add(editRefItem);
                }
            }
        }
        // adds a view source item
        JMenuItem viewSourceItem = new JMenuItem(VIEW_SOURCE_ITEM);
        viewSourceItem.addActionListener(popupListener);
        popupMenu.addSeparator();
        popupMenu.add(viewSourceItem);
        // adds a save item
        JMenuItem saveItem = new JMenuItem(SAVE_ITEM);
        saveItem.addActionListener(popupListener);
        popupMenu.add(saveItem);
        // shows the popup menu
        popupMenu.show(e.getComponent(), e.getX(), e.getY());
    }

}