/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.xerces;

import org.w3c.dom.Node;

import ch.cern.lhcb.xmleditor.gui.XmlTreeNode;

/**
 * This is an interface that every node displayed in the tree should implement
 * Since we only deal with this type of nodes, this replaces actually the Node
 * interface. Most of the methods declared here could be implemented once for
 * every class implementing this interface. They are actually duplicated in
 * every class due to the lack of multiple inheritance in java. If you do some
 * modification there, keep in mind that you could have to keep the different
 * implementations in sync.
 * 
 * @author Sebastien Ponce
 */
public interface TreeElement extends Node {

    /**
     * this method should be called whenever the name of this node is edited
     */
    public void nodeNameChanged();

    /**
     * this method should be called whenever the value of this node is edited
     */
    public void nodeValueChanged();

    /**
     * this method should be called when this node is removed
     */
    public void removed();

    /**
     * adds treeNode to the list of treeNode associated with this node
     * 
     * @param treeNode the TreeNode to add to the list
     */
    public void addTreeNode(XmlTreeNode treeNode);

    /**
     * removes treeNode from the list of treeNode associated with this node
     * 
     * @param treeNode the TreeNode to remove from the list
     */
    public void removeTreeNode(XmlTreeNode treeNode);

    /**
     * gets the value of this node, if it has one. The value of such a node is
     * in fact the value of its text child if it exists
     * 
     * @return the value of this node, null it has none
     */
    public String getValue();

    /**
     * sets the value of this node, if it has one. The value of such a node is
     * in fact the value of its text child if it exists
     * 
     * @param value the new value of this node
     */
    public void setValue(String value);

    /**
     * says whether node is an ancestor of this node or not. Take care that the
     * notion of ancestor does include the parent-child relation as well as the
     * element-attribute one. This means that an element is ancestor of its
     * attributes.
     * 
     * @param node the node to be tested
     * @return true if the current node is a descendant of node
     */
    public boolean isNodeAncestor(TreeElement node);

}