/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 * 
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this library; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 */

package com.simulogic.dataview.cursors;

import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.JSlider;
import javax.swing.ToolTipManager;

import com.simulogic.dataview.DataView;
import com.simulogic.dataview.ResourceManager;

/**
 * Slider with several thumbs to select regions of interest
 */
public class CursorSlider extends JSlider {

	protected Vector sliderModels;

	java.text.DecimalFormat format;

	private boolean locked;

	private boolean linked;

	private DataView view;

	/**
	 * Constructor
	 * 
	 * @param n
	 *            number of cursor thumbs
	 * @param v
	 *            parent data view
	 */
	public CursorSlider(int n, DataView v) {
		super();

		view = v;
		createThumbs(n);
		updateUI();

		setSnapToTicks(false);
		setPaintTrack(false);
		setOpaque(false);
		setAutoscrolls(true);

		format = new java.text.DecimalFormat("#.###");
		format.setMaximumFractionDigits(3);

		ToolTipManager.sharedInstance().registerComponent(this);
		addMouseMotionListener(view);
	}

	protected void createThumbs(int n) {
		sliderModels = new Vector(n);

		for (int i = 0; i < n; i++) {
			addThumb();
		}
	}

	/**
	 * @see javax.swing.JSlider#updateUI()
	 */
	public void updateUI() {
		setUI(new BasicCursorSliderUI(this));
	}

	/**
	 * Returns tooltip with information about cursor
	 * 
	 * @param e
	 *            mouse event
	 * @return tooltip string
	 */
	public String getToolTipText(MouseEvent e) {
		int thumbIndex = -1;

		for (int i = 0; i < getThumbNum(); i++) {
			Rectangle rect = ((CursorSliderAdditional) getUI()).getThumbRects()[i];
			if (rect.contains(e.getX(), e.getY())) {
				thumbIndex = i;
			}
		}

		if (thumbIndex == -1)
			return null;

		com.simulogic.dataview.DataChannel s = view.getStorage()
				.getSelectedChannel();

		String ux = "";
		String uy = "";

		String x = format.format(getValueAt(thumbIndex));
		String dx = "";
		ux = " " + view.getXCorner().getText();

		String y = ResourceManager.getResource("Unlocked");
		String dy = ResourceManager.getResource("Unlocked");

		if (locked && (s != null) && (s.getAttribute().isNormal())
				&& (!view.getStorage().isXChannel(s))) {
			x = format.format(getValueAt(thumbIndex));
			ux = " " + view.getStorage().getXChannel(s).getUnits();

			y = format.format(com.simulogic.dataview.analysis.ChannelStats
					.getValueAtX(getValueAt(thumbIndex), view.getStorage()
							.getXChannel(s), s));
			uy = " " + s.getUnits();
		}

		if (Math.IEEEremainder(thumbIndex, 2.0) != 0.0) {
			dx = dx
					+ format
							.format((getValueAt(thumbIndex) - getValueAt(thumbIndex - 1)));

			if (locked && (s != null) && (s.getAttribute().isNormal())
					&& (!view.getStorage().isXChannel(s))) {
				dy = format.format(com.simulogic.dataview.analysis.ChannelStats
						.getValueAtX(getValueAt(thumbIndex), view.getStorage()
								.getXChannel(s), s)
						- com.simulogic.dataview.analysis.ChannelStats
								.getValueAtX(getValueAt(thumbIndex - 1), view
										.getStorage().getXChannel(s), s));
			}
		}

		StringBuffer tip = new StringBuffer();
		tip.append("<p>" + "X: " + x + ux + "</p>");
		tip.append("<p>" + "Y: " + y + uy + "</p>");
		if (dx.length() != 0)
			tip.append("<p>" + "\u0394X: " + dx + ux + "</p><p>" + "\u0394Y: "
					+ dy + uy + "</p>");

		return "<html>" + "<center><strong>" + "Cursor " + (thumbIndex + 1)
				+ "</strong></center><hr>" + tip.toString() + " </html>";
	}

	/**
	 * Returns number of thumbs
	 * 
	 * @return number of thumbs
	 */
	public int getThumbNum() {
		if (sliderModels != null)
			return sliderModels.size();
		return 0;
	}

	/**
	 * Adds new thumb to th slider
	 */
	public void addThumb() {
		sliderModels.addElement(new DefaultDoubleBoundedRangeModel(50, 0, 0,
				100));

		updateUI();
	}

	/**
	 * Removes thumb from the slider
	 * 
	 * @param index
	 *            index of thumb to remove
	 */
	public void removeThumb(int index) {
		sliderModels.remove(index);

		updateUI();
	}

	/**
	 * Returns index of a partner cursor for the current cursor
	 * 
	 * @param index
	 *            thumb index
	 * @return partner thumb index
	 */
	public int getPartner(int index) {
		int partner = -1;

		if (Math.IEEEremainder(index, 2.0) != 0.0) {
			partner = index - 1;
		} else {
			partner = index + 1;
		}
		return partner;
	}

	/**
	 * Evenly distributes thumbs in the slider
	 * 
	 * @param min
	 *            min limit
	 * @param max
	 *            max limit
	 */
	public void distributeThumbs(double min, double max) {
		double step = Math.abs(max - min) / (getThumbNum() + 1.0);

		for (int i = 0; i < getThumbNum(); i++) {
			setValueAt(min + step * (i + 1.0), i);
		}
		fireStateChanged();
	}

	/**
	 * Returns value for the thumb
	 * 
	 * @param index
	 *            thumb index
	 * @return thumb value
	 */
	public double getValueAt(int index) {
		return getModelAt(index).getValue();
	}

	/**
	 * Sets value for the thumb
	 * 
	 * @param n
	 *            new value
	 * @param index
	 *            thumb index
	 */
	public void setValueAt(double n, int index) {
		if (getModelAt(index).getValue() != n) {

			if (isLinked()) {
				int linked = getPartner(index);
				double dx = getValueAt(index) - getValueAt(linked);

				if ((n - dx) > getMaximumLimit()
						|| (n - dx) < getMinimumLimit())
					return;

				getModelAt(linked).setValue(n - dx);
			}
			getModelAt(index).setValue(n);
			fireStateChanged();
		}
	}

	public int getMinimum() {
		return (int) this.getModelAt(0).getMinimum();
	}

	public double getMinimumLimit() {
		return this.getModelAt(0).getMinimum();
	}

	public void setMinimum(double min) {
		for (int i = 0; i < getThumbNum(); i++) {
			if (getModelAt(i).getMinimum() != min) {
				getModelAt(i).setMinimum(min);
			}
		}
		fireStateChanged();
	}

	public int getMaximum() {
		return (int) this.getModelAt(0).getMaximum();
	}

	public double getMaximumLimit() {
		return this.getModelAt(0).getMaximum();
	}

	public void setMaximum(double max) {
		for (int i = 0; i < getThumbNum(); i++) {
			if (getModelAt(i).getMaximum() != max) {
				getModelAt(i).setMaximum(max);
			}
		}
		fireStateChanged();
	}

	/**
	 * Returns model for index
	 * 
	 * @param index
	 *            index
	 * @return model
	 */
	public DoubleBoundedRangeModel getModelAt(int index) {
		return (DoubleBoundedRangeModel) sliderModels.get(index);
	}

	/**
	 * Returns locked status
	 * 
	 * @return true if locked
	 */
	public boolean isLocked() {
		return locked;
	}

	/**
	 * Sets locked status
	 * 
	 * @param locked
	 *            locked status
	 */
	public void setLocked(boolean locked) {
		this.locked = locked;
	}

	/**
	 * Returns linked status
	 * 
	 * @return true if linked
	 */
	public boolean isLinked() {
		return linked;
	}

	/**
	 * Sets linked status
	 * 
	 * @param linked
	 *            linked status
	 */
	public void setLinked(boolean linked) {
		this.linked = linked;
	}

	/**
	 * Returns parent data view
	 * 
	 * @return data view
	 */
	public DataView getView() {
		return view;
	}

}